
import pytest
from bluebook import Bluebook


def test_set_context__():
    class A(Bluebook):
        pass

    a = A()
    a.set_context__(name="PYLOT", live=True)

    assert a._context.get("name") == "PYLOT"
    assert a._context.get("live") is True

def test_extends__():

    class A(Bluebook):
        name = "A"

    class AA(Bluebook):
        v = "JJ"

    a = A()
    aa = AA()

    @a.extends__
    class B(object):
        CONST = "CONSTANT"
        def b(self):
            return "B"

    @a.extends__
    @aa.extends__  # Remember the order of the placement
    class C(object):
        def c(self):
            self.set_context__(WHO="WHO")
            return "C"

    @aa.extends__
    class D(object):
        def d(self):
            return "D"

    @a.extends__  # Extends a function
    def hello(self):
        return "HELLO"

    # Assert A
    assert a.b() == "B"
    assert a.c() == "C"
    assert a.hello() == "HELLO"
    assert a.CONST == "CONSTANT"
    assert a._context.get("WHO") == "WHO"

    # Assert AA
    assert aa.c() == "C"
    assert aa.d() == "D"

    # Exceptions
    with pytest.raises(AttributeError):
        # @aa didn't get attached to B, because it was called before @a was extended
        assert aa.CONST == "CONSTANT"
        assert aa.hello() == "HELLO"

def test_set_meta__():
    class A(Bluebook):
        def __init__(self):
            self.set_meta__(title="TITLE")
            self.set_meta__(description="DESCRIPTION")

        def get_meta(self, k):
            return self._context.get("META")[k]
    a = A()

    assert a.get_meta("title") == "TITLE"
    assert a.get_meta("description") == "DESCRIPTION"





"""
WebPortfolio command line tool

manage.py

Command line tool to manage your application

"""

import argparse
from application import get_config
import application.model as model
from webportfolio import utils

from webportfolio.module import user

config = get_config()
NAME = "WebPortfolio Manager"
__version__ = 1#config.APP_VERSION


def setup():

    # Create all db
    model.db.create_all()

    # :: USERS
    # Setup primary roles.
    # PRIMARY ROLES is a set of tuples [(level, name), ...]
    [model.User.Role.new(level=r[0], name=r[1]) for r in user.PRIMARY_ROLES]

    # ADD SUPER ADMIN
    email = config.ADMIN_EMAIL
    name = config.ADMIN_NAME
    if utils.is_valid_email(email):
        user = model.User.User.get_by_email(email)
        if not user:
            model.User.User.new(email=email, name=name, role="SUPERADMIN")
    else:
        raise AttributeError("Couldn't create new SUPERADMIN. 'email' is invalid")

    # :: POSTS
    # Set types
    post_types = ["Blog", "Article", "Page", "Other"]
    if not model.Cms.Type.all().count():
        [model.Cms.Type.new(t) for t in post_types]

    # Set categories
    post_categories = ["Blog"]
    if not model.Cms.Category.all().count():
        [model.Cms.Category.new(c) for c in post_categories]

    posts = [
        {
            "title": "About Us",
            "slug": "about",
            "type": "Page"
        },
        {
            "title": "Terms of Service",
            "slug": "tos",
            "type": "Page"
        }
    ]

def main():
    parser = argparse.ArgumentParser(description="%s  v.%s" % (NAME, __version__))
    parser.add_argument("--setup", help="Setup the system",  action="store_true")
    parser.add_argument("--upload-assets-to-s3", help="Upload all webportfolio files to S3", action="store_true")
    parser.add_argument("--assets", help="Assets",  action="store_true")
    arg = parser.parse_args()

    if arg.setup:
        # Default setup
        print("Setting up...")
        setup()

    if arg.upload_assets_to_s3:
        # Upload webportfolio files to s3
        from run_www import app
        #import flask_s3
        #import run_www  # Or the main application run file
        #print("Upload webportfolio files to S3")
        #flask_s3.create_all(run_www.app)
        print 1

    if arg.assets:

        print "Done"

from run_www import app
from flask.ext.script import Manager , Shell, Server
manager = Manager(app, with_default_commands=False)

@manager.command
def setup():
    """
    To setup a script
    """
    print "SETUP DONE!"

    print ("Hello World")

if __name__ == "__main__":
    from flask_assets import ManageAssets
    manager.add_command("assets", ManageAssets())

    manager.run()





"""
::WebPortfolio::

run_www.py

To run the server
"""

from webportfolio import WebPortfolio, get_env

# Import the application's views
import application.www.views

# The directory containing your views/webportfolio/templates
app_dir = "application/www"

# The project config object
app_config = "application.config.%s" % get_env()

WP = WebPortfolio()

# WebPortfolio.init returns Flask instance
app = WP.init(__name__, directory=app_dir, config=app_config, compress_html=False)


if __name__ == "__main__":
    import os
    import os.path as path
    extra_dirs = ['../webportfolio',]
    extra_files = extra_dirs[:]
    for extra_dir in extra_dirs:
        for dirname, dirs, files in os.walk(extra_dir):
            for filename in files:
                filename = path.join(dirname, filename)
                if path.isfile(filename):
                    extra_files.append(filename)
    app.run(debug=True, extra_files=extra_files)



"""
WebPortfolio

model.py

You may place your models here.
"""

from active_alchemy import SQLAlchemy
import config
from webportfolio import WebPortfolio, get_env_config
from webportfolio.module import user, cms

# The config
conf = get_env_config(config)

# Connect the DB
db = SQLAlchemy(conf.SQL_URI)

# Bind the DB connection to WebPortfolio
WebPortfolio.bind(db.init_app)

# ------------------------------------------------------------------------------

# User Model
User = user.model(db)

# Post Model
Cms = cms.model(User)

# A simple my not example table
class MyNote(db.Model):
    user_id = db.Column(db.Integer, db.ForeignKey(User.User.id))
    title = db.Column(db.String(250))
    content = db.Column(db.Text)
    user = db.relationship(User.User, backref="notes")



"""
::WebPortfolio::

https://github.com/mardix/webportfolio

serve_{project_name}.py

This is the entry point of the application.

--------------------------------------------------------------------------------

** To run the development serve

> webportfolio serve -p {project_name}

#---------

** To deploy with Propel ( https://github.com/mardix/propel )

> propel -w

#---------

** To deploy with Gunicorn

> gunicorn serve_{project_name}:app

"""

from webportfolio import WebPortfolio, get_env

# Import the application's views
import application.{project_name}.views


# 'app' is required if you intend to use WebPortfolio Cli
app = WebPortfolio.init(__name__, project="{project_name}")

